<?php

/*
 * ==========================================================
 * ULTIMATE MEMBERSHIP PRO APP
 * ==========================================================
 *
 * Ultimate Membership Pro App main file.  2017-2022 board.support. All rights reserved.
 *
 */

define('SB_UMP', '1.0.1');

/*
 * -----------------------------------------------------------
 * PANEL DATA
 * -----------------------------------------------------------
 *
 * Return the panel data for the right area of the admin conversations area
 *
 */

function sb_ump_get_conversation_details($user_id) {
    $response = ['subscriptions' => [], 'total' => 0];
    $user = sb_get_user($user_id);
    if (!empty($user['email'])) {
        $wp_user_id = sb_wp_get_user_id($user['email']);
        if (!empty($wp_user_id)) {
            $subscriptions = sb_ump_get_user_subscriptions($wp_user_id);
            $response['total'] = sb_ump_total_value($wp_user_id);
            $now = time();
            for ($i = 0; $i < count($subscriptions); $i++) {
                $subscriptions[$i]['expired'] = strtotime($subscriptions[$i]['expire_time']) < $now;
            }
            $response['subscriptions'] = $subscriptions;
            $response['currency_symbol'] = sb_get_setting('ump-currency-symbol', '');
        }
    }
    return $response;
}

/*
 * -----------------------------------------------------------
 * USERS
 * -----------------------------------------------------------
 *
 * 1. Return the registration fields
 * 2. Fired on user update to get the user extra values
 * 3. Return the user profile image
 * 4. Return the user extra values
 *
 */

function sb_ump_registration_fields() {
    $response = [];
    $fields = sb_wp_get_option('ihc_user_fields');
    $fields_exclude = ['user_login', 'user_email', 'confirm_email', 'first_name', 'last_name', 'pass1', 'pass2', 'description', 'country', 'tos', 'ihc_optin_accept', 'ihc_memberlist_accept', 'recaptcha', 'ihc_social_media', 'ihc_invitation_code_field', 'ihc_dynamic_price', 'ihc_coupon', 'payment_select', 'ihc_avatar'];
    if ($fields) {
        $fields = unserialize($fields);
        for ($i = 0; $i < count($fields); $i++) {
            if (!in_array($fields[$i]['name'], $fields_exclude)) {
                $active = false;
                if ($fields[$i]['display_admin'] == 1)
                    $active = true;
                elseif ($fields[$i]['display_public_reg'] == 1)
                    $active = true;
                elseif ($fields[$i]['display_public_ap'] == 1)
                    $active = true;
                elseif ($fields[$i]['display_on_modal'] == 1)
                    $active = true;
                array_push($response, ['slug' => $fields[$i]['name'], 'name' => $fields[$i]['label'], 'active' => $active]);
            }
        }
    }
    return $response;
}

function sb_ump_on_user_update() {
    $settings = [];
    $fields = sb_ump_registration_fields();
    for ($i = 0; $i < count($fields); $i++) {
        $slug = $fields[$i]['slug'];
        if (!empty($_POST[$slug])) {
            $settings[sb_string_slug($fields[$i]['name'])] = [$_POST[$slug], $fields[$i]['name']];
        }
    }
    return $settings;
}

function sb_ump_get_profile_image($id, $id_type = 'image') {
    if (empty($id))
        return '';
    $image = '';
    $image_id = $id_type == 'image' ? $id : sb_isset(sb_db_get('SELECT meta_value FROM wp_usermeta WHERE meta_key = "ihc_avatar" AND user_id = ' . $id), 'meta_value');
    if ($image_id) {
        if (function_exists('wp_get_attachment_image_src')) {
            $image = wp_get_attachment_image_src($id, 'large');
            $image = empty($image) ? '' : $image[0];
        } else {
            $image = sb_wp_get_image($image_id);
        }
    }
    return $image;
}

function sb_ump_get_user_extra($user_id) {
    $settings = [];
    $fields = sb_ump_registration_fields();
    $usermeta = sb_db_get('SELECT meta_value FROM ' . SB_WP_PREFIX . 'usermeta WHERE user_id = ' . $user_id);
    for ($i = 0; $i < count($fields); $i++) {
        $slug = $fields[$i]['slug'];
        if (!empty($usermeta[$slug])) {
            $settings[sb_string_slug($fields[$i]['name'])] = [$usermeta[$slug], $fields[$i]['name']];
        }
    }
    return $settings;
}

/*
 * -----------------------------------------------------------
 * MEMBERSHIP
 * -----------------------------------------------------------
 *
 * 1. Return the total value of the user
 * 2. Return all the user subscriptions
 * 3. Get the department ID linked to the active user membership
 * 4. Check if a user has at least one valid subscription
 *
 */

function sb_ump_total_value($user_id) {
    return sb_isset(sb_db_get('SELECT SUM(amount_value) AS sum, uid FROM ' . SB_WP_PREFIX . 'ihc_orders WHERE uid = ' . $user_id), 'sum', 0);
}

function sb_ump_get_user_subscriptions($user_id) {
    return sb_db_get('SELECT B.id, start_time, expire_time, label, price FROM ' . SB_WP_PREFIX . 'ihc_user_levels A, ' . SB_WP_PREFIX . 'ihc_memberships B WHERE A.level_id = B.id AND user_id = ' . $user_id, false);
}

function sb_ump_get_membership_department($user_id) {
    $departments_ump = sb_get_setting('ump-departments');
    if ($departments_ump && is_array($departments_ump)) {
        $subscriptions = sb_ump_get_user_subscriptions($user_id);
        if (!count($subscriptions)) {
            $subscriptions = [['id' => -1]];
        }
        for ($i = 0; $i < count($subscriptions); $i++) {
            for ($y = 0; $y < count($departments_ump); $y++) {
                if ($subscriptions[$i]['id'] == $departments_ump[$y]['membership-id']) {
                    return $departments_ump[$y]['department-id'];
                }
            }
        }
    }
    return false;
}

function sb_ump_is_member($user_id) {
    $subscriptions = sb_ump_get_user_subscriptions($user_id);
    $member = false;
    $free = true;
    $now = time();
    for ($i = 0; $i < count($subscriptions); $i++) {
        if (strtotime($subscriptions[$i]['expire_time']) > $now) {
            $member = true;
            if ($subscriptions[$i]['price'] != '0.00' && $subscriptions[$i]['price'] != '0') {
                $free = false;
                break;
            }
        }
    }
    return [$member, $free];
}

/*
 * ----------------------------------------------------------
 * # MORE FUNCTIONS
 * ----------------------------------------------------------
 *
 * 1. Check if the chat must be initialized
 *
 */

function sb_ump_is_init($user_id) {
    $init = sb_get_setting('ump-visibility');
    if (!$init) {
        return true;
    }
    $member = $user_id ? sb_ump_is_member($user_id) : [false];
    return $member[0] && ($init == 'members' || (!$member[1] && $init == 'paying-members'));
}

?>